# Encoding: Unicode UTF-8

# This R script generates a plot of the backbone torsion angles of nucleic acid
# and is made available under the following license terms:

# The MIT License (MIT)
# 
# Copyright (c) 2017 Markus Meier
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# AUTHOR:
# Markus Meier
# Department of Chemistry
# University of Manitoba
# 144 Dysart Road
# Winnipeg, MB, R3T 2N2, Canada
# E-mail: markus.meier "at" umanitoba.ca

# Version 1.0 (March 2017)


# Comments
  # Do not use 'data' as name, it is already taken by R.
  #
  # plot.default takes two arguments, 'panel.first' and 'panel.last', which are intented to be hook-in
  # functions for the plot. However this does not currently work. Replace
  # 'panel.first' with 'eval(panel.first)' and 'panel.last' with 'eval(panel.last)' in the plot.default
  # code in order to make it work.
  #
  # This example uses PDBID 2M4P (www.rcsb.org) which is an NMR ensemble with 10 models
  # To generate the JSON file, execute "x3dna-dssr -i=2m4p.pdb -o=2m4p.json --more --nmr --json"
  #
  # You can use the following code to record a chosen level of JSON hierarchy into a file for inspection
  #  sink('R_json_struct.txt')
  #  print(attr(myData, 'JSON'))
  #  sink()
  
# Setup R environment
  graphics.off()
  rm(list=ls())
  library('Cairo')
  library('plotrix')
  library('dplyr')
  library('tidyjson')
    
  # New plot and plot device
  
  # The Cairo convenience wrapper functions do not accept the 'units'
  # argument. If you need this, you have to rewrite them or 
  # call Cairo() directly.

  DEVICE <- 'pdf' # Graphics device for output:
                  # 'x11':    X-Windows environment (Unix, Cygwin)
                  # 'quartz': MacOS X
                  # 'win':    Windows
                  # 'pdf':    PDF output
                  # 'svg':    SVG output
  
  plotWidth <- 3.25  # by default in inches
  plotHeight <- 3.25 # by default in inches
  charSize <- 1.0 # par(cex=charSize)

  if(Sys.info()['sysname'] == 'Windows') {
    par(family='sans')    
  } else {
    CairoFonts(
      regular='Liberation Sans:style=Regular',
      bold='Liberation Sans:style=Bold',
      italic='Liberation Sans:style=Bold Oblique',
      bolditalic='Liberation Sans:style=Bold Oblique',
      symbol='Symbol:Regular')
  }
  if(DEVICE == 'pdf') {
    CairoPDF(file='backbone_torsions_2m4p.pdf', width=plotWidth, height=plotHeight)
  } else if(DEVICE == 'svg') {
    CairoSVG(file='backbone_torsions_2m4p.svg', width=plotWidth, height=plotHeight, bg='white')
  } else if(DEVICE == 'x11') {
    CairoX11(width=plotWidth, height=plotHeight)
    # the following work-around fixes a Cairo quirk
    # (default window does not automatically refresh)
    dev.new(width=plotWidth, height=plotHeight) 
    dev.off(2)
  } else if(DEVICE == 'quartz') {
    quartz(width=plotWidth, height=plotHeight, family='Liberation Sans')
  } else if(DEVICE == 'win') {
    CairoWin(width=plotWidth, height=plotHeight)
  }

# ************************ ENTER YOUR STUFF HERE *******************************
  
  # Read JSON file generated by x3dna-dssr
  myData <- read_json('2m4p.json', format='json') # returns tbl_json
#  sink('R_json_struct.txt')
#  print(attr(myData, 'JSON'))
#  sink()
  
  # Navigate through the data structure
  # x3dna-dssr generates a different depth of JSON data structure,
  # depending if the --nmr option is used or not
  
  # For JSON generated with --nmr option, use this code:
  nts_array <- myData  %>%
    enter_object('models') %>%
    gather_array() %>%
    enter_object('parameters') %>%
    enter_object('nts') %>%
    gather_array()
    
  # For JSON generated without the --nmr option, use this code, instead:
#  nts_array <- myData  %>%
#    enter_object('nts') %>%
#    gather_array()

#  sink('R_json_struct_nts_array.txt')
#  print(attr(nts_array, 'JSON'))
#  sink()

  # Now we have collapsed the JSON data enough that we can extract what we want:
  
  tidyData <- nts_array %>% 
    spread_values(chain = jstring('chain_name')) %>%
    spread_values(id = jstring('nt_id')) %>% 
    spread_values(resi = jstring('nt_resnum')) %>% 
    spread_values(resn = jstring('nt_name')) %>% 
    spread_values(alpha = jnumber('alpha')) %>% 
    spread_values(beta = jnumber('beta')) %>% 
    spread_values(gamma = jnumber('gamma')) %>% 
    spread_values(delta = jnumber('delta')) %>% 
    spread_values(epsilon = jnumber('epsilon')) %>% 
    spread_values(zeta = jnumber('zeta')) %>% 
    spread_values(chi = jnumber('chi')) %>%
    spread_values(baseSugar_conf = jstring('baseSugar_conf')) %>%
    spread_values(sugar_class = jstring('sugar_class'))
     
  # Now we can use dplyr's filter to extract the nucleotides we are interested in:
  G4_all <- tidyData %>% filter(resn=='DG')
  G4_flankBulge <- G4_all %>% filter(resi==3 | resi==5)
  G4_others <- G4_all %>% filter(!(resi==3 | resi==5))
  G4_resi3 <- G4_all %>% filter(resi==3)
  G4_resi5 <- G4_all %>% filter(resi==5)
  
  # Plotting with polar.plot()
  # rp.type:
  # r: radial lines
  # s: symbols
  # p: polygons, points defined by 'length'/'polar.pos' argument are connected by line segments
  # add=TRUE: add more graphics to an existing plot
  # add=FALSE: replace the existing plot
  # radial.lim=c(centre, ring, ring, .., periphery), each value produces a ring at specified radial distance
  #   This argument to be given to each subsequent call to polar.plot(add=TRUE)

  main <- 'PDBID 2M4P (all conformers)'
  # Bulge: T4
  
  show.grid.labels <- 4 # 0: no labels
                        # 1: labels at 270°
                        # 2: labels at 180°
                        # 3: labels at 90°
                        # 4: labels at 0°

  labels <- c('0°', '45°', '90°', '135°', '180°', '-135°', '-90°', '-45°')
  label.pos <- c(0.0, 45.0, 90.0, 135.0, 180.0, 225.0, 270.0, 315.0)

  show.radial.grid <- FALSE
  alpha.pos <- 0.25
  beta.pos <- 0.4
  gamma.pos <- 0.55
  delta.pos <- 0.7
  epsilon.pos <- 0.85
  zeta.pos <- 1.0
  radial.lim <- c(0.0, alpha.pos , beta.pos, gamma.pos, delta.pos, epsilon.pos, zeta.pos)
  radial.labels <- c('', 'α', 'β', 'γ', 'δ', 'ε', 'ζ')
  point.symbols <- 1
  point.col <- 'black'

  # Some available point symbols
  # 1  circle
  # 2  triangle up
  # 3  cross
  # 4  saltire
  # 5  diamond
  # 6  downwards triangle
  # 7  boxed saltire
  # 8  star
  # 9  diamond boxed cross
  # 10 circled cross
  # 11 double triangle
  # 12 boxed cross
  # 13 circled saltire
  # 14 boxed upwards triangle
  # 15 filled rectangle
  # 16 filled circle
  # 17 filled upwards triangle
  # 18 filled diamond
  # 26 no symbol (ignore warning).
  
  polar.plot(alpha.pos, G4_others$alpha,
    rp.type='s', point.symbols=point.symbols, radial.lim=radial.lim, mar=c(2,2,2,2),
    labels=labels, label.pos=label.pos, label.prop=1.23, show.radial.grid=show.radial.grid, 
    show.grid.labels=show.grid.labels, radial.labels=radial.labels, boxed.radial=FALSE, grid.left=FALSE)

  par(font=2)
  mtext(main, side=1, line=0.9, at=-1.4, adj=0)
  
  polar.plot(beta.pos, G4_others$beta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(gamma.pos, G4_others$gamma, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(delta.pos, G4_others$delta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(epsilon.pos, G4_others$epsilon, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(zeta.pos, G4_others$zeta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  
  # plot in blue
  point.symbols <- 5
  point.col <- 'mediumorchid'
  polar.plot(alpha.pos, G4_resi3$alpha, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(beta.pos, G4_resi3$beta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(gamma.pos, G4_resi3$gamma, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(delta.pos, G4_resi3$delta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(epsilon.pos, G4_resi3$epsilon, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(zeta.pos, G4_resi3$zeta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  
  # plot in red
  point.symbols <- 2
  point.col <- 'seagreen2'
  polar.plot(alpha.pos, G4_resi5$alpha, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(beta.pos, G4_resi5$beta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(gamma.pos, G4_resi5$gamma, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(delta.pos, G4_resi5$delta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(epsilon.pos, G4_resi5$epsilon, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  polar.plot(zeta.pos, G4_resi5$zeta, rp.type='s', point.symbols=point.symbols, point.col=point.col, radial.lim=radial.lim, add=TRUE)
  
  par(col='mediumorchid')
  text(-1.3, 1.2, 'G3')
  par(col='seagreen2')
  text(-1.3, 1.0, 'G5')
  
  if(DEVICE=='pdf' | DEVICE=='svg') {
    dev.off()
  }
 
